voice_dBFS_diff = 2
voice_dBFS_sample_interval = 100
voice_dBFS_human_var_min = 5
voice_dBFS_human_max = -10
voice_recognition_div_human_min = 0.5
voice_ppl_max_length = 15
voice_ppl_min_length = 5
voice_modal_particles_pattern = "啊|嗯|呃|呢|这个|那个|吧|嘛|呀"
voice_modal_particles_pattern_pinyin = " a | n2 | e4 | ne | zhe4 ge4 | na4 ge4 | ba | ma | ya "
voice_length_per_count_min = 10
voice_length_per_count_max = 50
voice_recognition_duration_per_pause_count_min = 10
voice_recognition_duration_per_pause_count_max = 50
voice_pause_duration_ratio_min = 0
voice_pause_duration_ratio_max = 0.4
voice_repeat_one_count_min = 0.005
voice_repeat_one_count_max = 0.02
skip_word4ppl = ['我叫', '岁', '我是']
start_word4ppl = ['然后', '最后', '接着']
text_clean_for_ppl = "啊|嗯|呃|！|，|？|。"
text_clean_for_modal_particles = "！|，|？|。"
text_clean = u"([^\u4e00-\u9fa5\u0030-\u0039\u0041-\u005a\u0061-\u007a])"
KEY_FRAME_INTERVAL = 1
heart_rate_duration = 1000
heart_rate_windows = 1



# -*- coding: utf-8 -*-
import Levenshtein as Lev


def wer(s1, s2):
    """
    Computes the Word Error Rate, defined as the edit distance between the
    two provided sentences after tokenizing to words.
    Arguments:
        s1 (string): space-separated sentence
        s2 (string): space-separated sentence
    """
    # build mapping of words to integers
    b = set(s1.split() + s2.split())
    word2char = dict(zip(b, range(len(b))))
    # print(word2char)
    # map the words to a char array (Levenshtein packages only accepts
    # strings)
    w1 = [chr(word2char[w]) for w in s1.split()]
    w2 = [chr(word2char[w]) for w in s2.split()]

    return Lev.distance(''.join(w1), ''.join(w2))/len(s1.split())



import re
import numpy as np
from pydub import AudioSegment
from pypinyin import pinyin as get_pinyin, Style
from mmanalyser.utils.wer import wer
from mmanalyser.utils.number_to_chinese import number_to_chinese
from mmanalyser.utils.vad import vad_robust
from mmanalyser import config
import logging

WORD_SEP = '#'
AUDIO_START_TIME = 500


class ExpressionAnalyser:
    '''
    表达能力分析器

    asr数据格式：{"pause_detection": [[3280, 3922], [7781, 8688]], "text_corrected":  [], "text":  [[[0, 1170], "领导你好，"],
                                     [[8940, 12250], "我在嘉兴，"], [[12250, 13490], "有两家造纸厂，"]]}
    '''
    def __init__(self, rationality_service_url=None):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.ntc = number_to_chinese()
        self.middle_value = dict()

    def get_expression_score(self, asr, pinyin, audio_path, use_pinyin=False):
        """
        获取表达能力分数

        Args:
            asr: asr结果
            audio_path:  音频地址

        Returns:
             dict: {
            'talk_speed': float,        // 语速
            'audio_clear': bool,        //声音是否清晰
            'mandarin_score': float,    //普通话标准程度
            'expression_score': float,  //表达能力分数
            }
        """
        self.middle_value['path'] = audio_path
        raw_text = ''.join([x[1] for x in asr['text']])
        recognition_duration, pause_moment = self.__get_duration_information(
            audio_path)
        text, text_length, particle_count, repeat_one_count, ranhou_count = self.__disfluency_detection(
            raw_text, pinyin, use_pinyin)
        mandarin_score = self.__parse_mandarin_score(text, pinyin)
        talk_speed = self.__get_talk_speed(text_length, recognition_duration)
        audio_clear = self.__is_audio_clear(audio_path, text_length,
                                            recognition_duration,
                                            mandarin_score)
        express_score = self.__parse_expression_score(
            audio_clear, text_length, pause_moment, recognition_duration,
            particle_count, mandarin_score, repeat_one_count, ranhou_count)
        self.middle_value.update({
            'talk_speed': talk_speed,
            'audio_clear': audio_clear,
            'mandarin_score': mandarin_score,
            'expression_score': express_score,
        })
        return {
            'talk_speed': talk_speed,
            'audio_clear': audio_clear,
            'mandarin_score': mandarin_score,
            'expression_score': express_score,
        }

    def get_talk_speed(self, asr):
        '''
        获取语速指标

        Args:
            asr: asr 结果
        Returns:
           float: 语速
        '''
        raw_text = ''.join([x[1] for x in asr['text']])
        recognition_duration, pause_moment = \
            self.__get_duration_information(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = \
            self.__disfluency_detection(raw_text)
        talk_speed = self.__get_talk_speed(text_length, recognition_duration)
        return talk_speed

    def get_mandarin_score(self, asr, pinyin):
        '''
        获取普通话标准程度

        Args:
            asr: asr结果
            pinyin: 拼音识别结果（"wo2 ai2 ni3"）
        Returns:
            float: 普通话标准程度
        '''

        raw_text = ''.join([x[1] for x in asr['text']])
        text = re.sub(config.text_clean, '', raw_text)
        return self.__parse_mandarin_score(text, pinyin)

    def get_audio_clear(self, asr, audio_path, pinyin):
        '''
        获取声音是否清晰

        Args:
            asr: asr结果
            audio_path: 音频地址

        Returns:
            bool: 清晰返回true 不清晰返回false
        '''
        raw_text = ''.join([x[1] for x in asr['text']])
        recognition_duration, pause_moment = \
            self.__get_duration_information(asr)
        text, text_length, particle_count, repeat_one_count, ranhou_count = \
            self.__disfluency_detection(raw_text)
        mandarin_score = self.__parse_mandarin_score(text)
        audio_clear = self.__is_audio_clear(audio_path, text_length,
                                            recognition_duration,
                                            mandarin_score)
        return audio_clear

    def __get_talk_speed(self, text_length, recognition_duration):
        self.logger.debug("get talk speed")
        return text_length / recognition_duration

    def __is_audio_clear(self, audio_path, length, recognition_duration,
                         mandarin_score):
        self.logger.debug("get audio clear")
        flag = (
            mandarin_score != 0 and recognition_duration > 0 and
            (length / recognition_duration > 0.75 or
             (length / recognition_duration < 0.75 and mandarin_score > 50)))
        if audio_path is not None:
            dBFS_var, dBFS_mean = self.__get_dBFS_information(audio_path)
            return bool(dBFS_mean < config.voice_dBFS_human_max
                        and dBFS_var > config.voice_dBFS_human_var_min
                        and flag)
        else:
            return flag

    def __parse_mandarin_score(self, text, pinyin):
        self.logger.debug("parse mandarin score")

        def func(x):
            #return x
            if x > 1.2:
                return 0.15 / x
            elif x < 0.4:
                return 0.1 * (0.4 - x) + 0.95
            else:
                return (1.2 - x) / 0.8 * 0.85 + 0.10

        text = self.ntc.parse(text)
        text = ' '.join(
            [x[0] for x in get_pinyin(str(text), style=Style.TONE3)])
        return func(wer(text, pinyin)) * 100

    def __parse_expression_score(self, audio_clear, text_length, pause_moment,
                                 recognition_duration, particle_count,
                                 mandarin_score, repeat_one_count,
                                 ranhou_count):
        self.logger.debug("get expression score")
        if not audio_clear:
            return 0

        if len(pause_moment) == 0:
            recognition_duration_per_pause_count = config.voice_recognition_duration_per_pause_count_max
        else:
            recognition_duration_per_pause_count = text_length / \
                len(pause_moment)

        recognition_duration_per_pause_count_score = self.__min_max_scaler(
            recognition_duration_per_pause_count,
            config.voice_recognition_duration_per_pause_count_min,
            config.voice_recognition_duration_per_pause_count_max)
        pause_duration_ratio = sum(
            pause_moment
        ) / recognition_duration if recognition_duration > 0 else 1
        pause_duration_ratio_score = self.__min_max_scaler(
            pause_duration_ratio,
            config.voice_pause_duration_ratio_min,
            config.voice_pause_duration_ratio_max,
            negative=True)

        # 语气词 0~1分
        if particle_count == 0:
            length_per_count = config.voice_length_per_count_max
        else:
            length_per_count = text_length / particle_count
        length_per_count_score = self.__min_max_scaler(
            length_per_count, config.voice_length_per_count_min,
            config.voice_length_per_count_max)

        ranhou_count_per_length = ranhou_count / text_length
        ranhou_count_per_length_score = self.__min_max_scaler(
            ranhou_count_per_length, 0.02, 0.06)

        repeat_one_count_per_length = repeat_one_count / text_length
        repeat_one_count_score = self.__min_max_scaler(
            repeat_one_count_per_length,
            config.voice_repeat_one_count_min,
            config.voice_repeat_one_count_max,
            negative=True)
        decay_factor = 0.8 if pause_duration_ratio_score < 0.1 or \
            recognition_duration_per_pause_count_score < 0.1 or \
            length_per_count_score < 0.1 or \
            repeat_one_count_score < 0.1 else 1
        num = int(pause_duration_ratio_score > 0.9) + int(
            recognition_duration_per_pause_count_score > 0.9) + int(
                length_per_count_score > 0.9) + int(
                    repeat_one_count_score > 0.9)
        self.middle_value.update(
            dict(recognition_duration_per_pause_count=
                 recognition_duration_per_pause_count,
                 pause_duration_ratio=pause_duration_ratio,
                 ranhou_count_per_length=ranhou_count_per_length,
                 length_per_count=length_per_count,
                 repeat_one_count_per_length=repeat_one_count_per_length))
        return (0.5 * pause_duration_ratio_score +
                0.5 * recognition_duration_per_pause_count_score +
                1.5 * length_per_count_score +
                1.5 * repeat_one_count_score) * decay_factor * (
                    0.85 + 0.1 * mandarin_score /
                    100) * 20 + 5 * num - ranhou_count_per_length_score * 10


#     def __parse_expression_score(
#             self,
#             audio_clear,
#             text_length,
#             pause_moment,
#             recognition_duration,
#             particle_count,
#             mandarin_score,
#             repeat_one_count,
#             ranhou_count):
#         self.logger.debug("get expression score")
#         if not audio_clear:
#             return 0

#         if len(pause_moment) == 0:
#             recognition_duration_per_pause_count = config.voice_recognition_duration_per_pause_count_max
#         else:
#             recognition_duration_per_pause_count = text_length / \
#                 len(pause_moment)

#         pause_duration_ratio = sum(pause_moment) / recognition_duration if recognition_duration > 0 else 1
#         # 语气词 0~1分
#         if particle_count == 0:
#             length_per_count = config.voice_length_per_count_max
#         else:
#             length_per_count = text_length / particle_count

#         ranhou_count_per_length = ranhou_count / text_length
#         repeat_one_count_per_length = repeat_one_count / text_length
#         self.middle_value.update(dict(recognition_duration_per_pause_count=recognition_duration_per_pause_count,
#                                       pause_duration_ratio=pause_duration_ratio,
#                                       ranhou_count_per_length=ranhou_count_per_length,
#                                       length_per_count=length_per_count, repeat_one_count_per_length=repeat_one_count_per_length))

#         recognition_duration_per_pause_count_score = self.__min_max_scaler(
#             recognition_duration_per_pause_count,
#             config.voice_recognition_duration_per_pause_count_min,
#             config.voice_recognition_duration_per_pause_count_max)

#         pause_duration_ratio_score = self.__min_max_scaler(
#             pause_duration_ratio,
#             config.voice_pause_duration_ratio_min,
#             config.voice_pause_duration_ratio_max,
#             negative=True)

#         length_per_count_score = self.__min_max_scaler(
#             length_per_count, config.voice_length_per_count_min,
#             config.voice_length_per_count_max)
#         repeat_one_count_per_length_score = self.__min_max_scaler(
#             repeat_one_count_per_length,
#             config.voice_repeat_one_count_min,
#             config.voice_repeat_one_count_max,
#             negative=True)
#         ranhou_count_per_length_score = self.__min_max_scaler(
#             ranhou_count_per_length, 0.02, 0.06, negative=True)
#         decay_factor = 0.8 if pause_duration_ratio_score == 0 or \
#             recognition_duration_per_pause_count_score == 0 or \
#             length_per_count_score == 0 or \
#             repeat_one_count_per_length_score == 0 else 1
#         print(pause_duration_ratio_score, recognition_duration_per_pause_count_score, length_per_count_score,
#               repeat_one_count_per_length_score, ranhou_count_per_length_score, mandarin_score)
#         return (1 * pause_duration_ratio_score + 1.25 *
#                 recognition_duration_per_pause_count_score + 1.75 * length_per_count_score +
#                 0.5 * repeat_one_count_per_length_score + ranhou_count_per_length_score * 0.5) * decay_factor * (0.8 + 0.15 * mandarin_score/100)

    def __get_duration_information(self, audio_path):
        result = vad_robust(audio_path)[1]
        recognition_duration = sum([x[1] - x[0] for x in result])
        # 停顿
        moment = []
        for x in result:
            moment.append(x[0])
            moment.append(x[1])
        pause_moment = []
        for x in range(2, len(moment), 2):
            duration = (moment[x] - moment[x - 1])
            if duration > 0.5:
                pause_moment.append(duration)
        self.middle_value['recognition_duration'] = recognition_duration
        self.middle_value['pause_moment'] = pause_moment
        return recognition_duration, pause_moment

    def __disfluency_detection(self, text, pinyin, use_pinyin=False):
        text = re.sub(config.text_clean, '', text)
        text = re.sub('[a-zA-Z]', '', text)
        if not use_pinyin:
            # 语气词
            particle_count = len(
                re.findall(config.voice_modal_particles_pattern, text))
            # 文本清理
            repeat_one_count = len(self.__get_repeat_record_by_offset(text, 1))
            ranhou_count = len(re.findall('然后', text))
            text_length = len(text)
        else:
            # 语气词
            particle_count = len(
                re.findall(config.voice_modal_particles_pattern_pinyin,
                           ' ' + pinyin + ' '))
            # 文本清理
            repeat_one_count = len(
                self.__get_repeat_record_by_offset(pinyin, 1))
            ranhou_count = len(re.findall(' ran2 hou4 ', pinyin))
            text_length = len(pinyin.split())

        self.middle_value['text_length'] = text_length
        self.middle_value['text'] = text
        self.middle_value['ranhou_count'] = ranhou_count
        self.middle_value['repeat_one_count'] = repeat_one_count
        self.middle_value['particle_count'] = particle_count
        return text, text_length, particle_count, repeat_one_count, ranhou_count

    def __get_dBFS_information(self, audio):
        sound = AudioSegment.from_file(audio, "mp3")
        dBFS = []
        for x in range(AUDIO_START_TIME, len(sound),
                       config.voice_dBFS_sample_interval):
            dbfs = sound[x:x + config.voice_dBFS_sample_interval].dBFS
            if dbfs != float("-inf"):
                dBFS.append(dbfs)
        return np.var(dBFS), np.mean(dBFS)

    def __min_max_scaler(self, value, min_value, max_value, negative=False):
        score = 0
        if value > max_value:
            score = 1
        elif value > min_value:
            diff = value - min_value
            score += diff / (max_value - min_value)
        return score if not negative else 1 - score

    def __get_repeat_record_by_offset(self, text, offset=1):
        assert offset > 0
        length = len(text)
        repeat_record = []
        word = text[0:offset] if length > 0 else ''
        i = offset * 2
        count = 1
        while i < length + 1:
            if word == text[i - offset:i]:
                count += 1
            else:
                if count > 1:
                    if word + word not in ['勉勉', '强强', '谢谢']:
                        repeat_record.append(count)
                i = i - offset + 1
                count = 1
                word = text[i - offset:i]
            i = i + offset
        if count > 1:
            if word + word not in ['勉勉', '强强', '谢谢']:
                repeat_record.append(count)
        return repeat_record

    def __get_repeat_record_pinyin_by_offset(pinyin, offset=1):
        assert offset > 0
        data = pinyin.split(' ')
        pinyin2idx = {y: chr(x) for x, y in enumerate(data)}
        # idx2pinyin = {y:x for x, y in pinyin2idx.items()}
        text = ''.join([pinyin2idx[x] for x in data])
        length = len(text)
        repeat_record = []
        word = text[0:offset] if length > 0 else ''
        i = offset * 2
        count = 1
        while i < length + 1:
            if word == text[i - offset:i]:
                count += 1
            else:
                if count > 1:
                    repeat_record.append(count)
                i = i - offset + 1
                count = 1
                word = text[i - offset:i]
            i = i + offset
        if count > 1:
            repeat_record.append(count)
        return repeat_record

if __name__ == '__main__':
    ea = ExpressionAnalyser()
    fake_asr = {
        "pause_detection": [[3280, 3922], [7781, 8688]],  # 停顿检测结果
        "text": [[[0, 1170], "领导你好，"], [[1170, 2370], "我叫秦嘉涵，"],
                 [[2370, 5530], "今年26岁，"], [[5530, 8040], "是浙江龙游人，"],
                 [[8040, 8940], "我，"], [[8940, 12250], "我在龙游，"],
                 [[12250, 13490], "有两家造纸厂，"], [[13490, 19330], "有4年的维修工禁令。"]]
    }
    fake_pinyin = ' '.join(
        [x[0] for x in get_pinyin("领导你好", style=Style.TONE3)])
    print(
        ea.get_expression_score(asr=fake_asr,
                                pinyin=fake_pinyin,
                                audio_path='temp/10.wav'))
    print(ea.middle_value)
    print(
        ea.get_expression_score(asr=fake_asr,
                                pinyin=fake_pinyin,
                                audio_path='temp/10.wav',
                                use_pinyin=True))
    print(ea.middle_value)
    print(ea.get_mandarin_score(asr=fake_asr, pinyin=fake_pinyin))



import setuptools
import mmanalyser as package


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mmanalyser",
    version=package.__version__,
    author="xujc",
    author_email="",
    description="a tool for multimedia",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    install_requires=[
        "pydub==0.23.1",
        "pandas==0.25.3",
        "etools==0.0.10",
        "pyAudioAnalysis==0.3.0",
        "webrtcvad==2.0.10",
        "python-Levenshtein==0.12.0",
        "pypinyin==0.37.0"

    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    package_data={'mmanalyser': ['data/*', 'utils/*']}
)



import numpy as np
from mmanalyser import config
from mmanalyser.utils.suffix_remover import suffix_remover
import logging


class FluctuationDetector:
    '''
    波动检测器
    '''
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def get_abnormal_fluctuation_range_from_heart_rate(self, heart_rate_infos):
        '''
        从心率信息中抽取异常波动时间范围

        Args:
            heart_rate_infos: [[70, 1000.0], [75, 2000.0], ...]

        Returns:
            list: [start_time, end_time]
        '''
        # 取第10s到第15s区间的心率均值作为基准
        bpm_baseline = np.mean(
            [x[0] for x in heart_rate_infos if 10000 <= x[1] <= 15000])
        if np.isnan(bpm_baseline):
            bpm_baseline = 70.0
        # 找出超过基准心率 25% 以上，且波动最大的区间
        thresh = bpm_baseline * 1.25
        significant_info = [0, 0, 0]
        sig = False
        accum_bpm = 0
        count = 0
        for x in heart_rate_infos:
            if x[0] > thresh:
                accum_bpm += x[0]
                count += 1
                if not sig:
                    start = x[1]
                    sig = True
            elif x[0] <= thresh and sig:
                end = x[1]
                sig = False
                mean_bpm = accum_bpm / float(count)
                accum_bpm = 0
                count = 0
                # 持续时间超过 1s 才记录
                if end - start > 1000 and mean_bpm > significant_info[2]:
                    significant_info = [start, end, mean_bpm]
        return significant_info[:2]

    def get_abnormal_fluctuation_range_from_emotion_fluction(self, emotion_fluctuation):
        '''
        从情绪波动信息中抽取异常波动时间范围

        Args:
            emotion_fluctuation: [0.2625822317841726, 0.286318327656083, 0.19226654361134174, 0.26339393328225225, 0.3021995866171042, 0.2843571171657221, ...]

        Returns:
            list: [start_time, end_time]

        '''
        fluctuation_abs = list(map(abs, emotion_fluctuation))
        max_val = max(fluctuation_abs)
        emotion_change_threshold = 0.9
        if max_val > emotion_change_threshold:
            start = fluctuation_abs.index(max_val)
            return (start * config.KEY_FRAME_INTERVAL * 1000,
                    (start + 1) * config.KEY_FRAME_INTERVAL * 1000)
        return None

    def get_text_from_emotion_abnormal_fluction(self, text_with_timestamp, emotion_fluctuation):
        '''
        结合带时间戳的文本抽取当情绪波动异常时的文本

        Args:
            text_with_timestamp: [[[[0, 1170], "领导你好，"], [[1170, 2370], "我叫，"]], ...]
            emotion_fluctuation: [0.2625822317841726, 0.286318327656083, 0.19226654361134174, 0.26339393328225225, 0.3021995866171042, 0.2843571171657221, ...]

        Returns
            str: "abnormal_text"
        '''
        emotion_range = self.get_abnormal_fluctuation_range_from_emotion_fluction(emotion_fluctuation)
        return self.__get_text_from_range(text_with_timestamp, emotion_range)

    def get_text_from_heart_rate_abnormal_fluction(self, text_with_timestamp, heart_rate):
        '''
        结合带时间戳的文本抽取心率波动异常时的文本

        Args:
            text_with_timestamp: [[[[0, 1170], "领导你好，"], [[1170, 2370], "我叫，"]], ...]
            heart_rate: [[70, 1000.0], [75, 2000.0], ...]

        Returns:
            str: "abnormal_text"
        '''
        heart_rate_range = self.get_abnormal_fluctuation_range_from_heart_rate(heart_rate)
        return self.__get_text_from_range(text_with_timestamp, heart_rate_range)

    def __get_text_from_range(self, text_with_timestamp, range):
        abnormal_text = ''
        if text_with_timestamp and range:
            for r, text in text_with_timestamp:
                if self.__overlap(r, range):
                    abnormal_text += text
        return suffix_remover.remove_suffix(abnormal_text)

    def __overlap(self, r1, r2):
        return r2[0] <= r1[0] <= r2[1] or r2[0] <= r1[1] <= r2[1] or \
               r1[0] <= r2[0] <= r1[1] or r1[0] <= r2[1] <= r1[1]


if __name__ == '__main__':
    fake_heart_rate_fluction = [72.57492507492508, 92.03153144697094, 78.93288496016338, 84.63336143314451, 87.28089096665917, 86.75675675675674, 97.29729729729729, 104.59459459459457, 76.2162162162162, 60.81081081081082, 60.81081081081082, 60.81081081081082, 60.81081081081082, 60.81081081081082, 77.02702702702703, 80.27027027027025, 72.97297297297295, 67.2972972972973, 94.86486486486487, 81.08108108108108]
    text = [[[0, 1170], "领导你好，"], [[1170, 2370], "我叫，"], [[2370, 5530], "今年26岁，"], [[5530, 8040], "是浙江人，"], [[8040, 8940], "我，"], [[8940, 12250], "我在龙游，"], [[12250, 13490], "有两家造纸厂，"], [[13490, 19330], "有4年的维修工禁令。"]]
    fa = FluctuationDetector()
    print(fa.get_text_from_emotion_abnormal_fluction(text_with_timestamp=text, emotion_fluctuation=fake_emotion_fluction))




# -*- coding: utf-8 -*-
import numpy as np
from mmanalyser.utils.vad import vad_robust
from pyAudioAnalysis import audioTrainTest as aT
import logging
import os
import subprocess
import shutil


class AudioAnalyser:
    '''
    音频分析器

    '''
    def __init__(self, rationality_service_url=None):
        self.logger = logging.getLogger(self.__class__.__name__)

    def is_valid_audio(self, audio_path, human_voice_length=4):
        pred = self.is_human_voice(audio_path)
        return pred > 0.5, pred

    def is_human_voice(self, audio_path, human_voice_length=1, mode=0):
        pred = 0
        if mode == 0:
            paths, voiced_range, voiced_duration, duration = vad_robust(
                audio_path, span=3, chunk_dir='temp/')
            if voiced_duration >= human_voice_length:
                pred = float(
                    np.mean(
                        [self.is_human_voice_snippet(x) for x in paths[1:]]))
        elif mode == 1:
            paths, voiced_range, voiced_duration, duration = vad_robust(
                audio_path, chunk_dir='temp/')
            if voiced_duration >= human_voice_length:
                haha = np.array([[idx, x[0][1] - x[0][0]]
                                for idx, x in enumerate(voiced_range)
                                if x[0][1] - x[0][0] > 1])
                if len(haha) > 0:
                    haha[:, 1] = haha[:, 1] / np.sum(haha[:, 1])
                    pred = sum([
                        self.is_human_voice_snippet(paths[1 + int(idx)]) * ratio
                        for idx, ratio in haha
                    ])

        chunk_dir = os.path.join('temp/', os.path.basename(audio_path)[:-4])
        shutil.rmtree(chunk_dir)
        return pred

    def is_human_voice_snippet(self, audio_path):
        pred = aT.file_classification(
            audio_path,
            os.path.join(os.path.dirname(os.path.dirname(__file__)),
                         'mmanalyser', 'data', "gbdt"),
            "gradientboosting")[1][0]
        return pred

    def delete_silence_snippet(self, audio_path, chunk_dir='temp/'):
        if chunk_dir is None:
            chunk_dir = 'temp/'
        return vad_robust(audio_path, chunk_dir=chunk_dir)

    def convert_to_wav(self,
                       source_path,
                       desc_path,
                       channel=1,
                       sample_rate=16000):
        cmd = [
            "ffmpeg", "-i", f"{source_path}", "-f", "wav", "-ar",
            str(sample_rate), "-ac",
            str(channel), f"{desc_path}"
        ]
        if os.path.exists(desc_path):
            os.remove(desc_path)
        self.logger.info(' '.join(cmd))
        obj = subprocess.Popen(cmd,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               stdin=subprocess.PIPE)
        status_code = obj.wait()
        return status_code



# -*- coding:utf-8 -*-
import logging
import os
from etools.singleton_meta import SingletonMeta


class SuffixRemover(object, metaclass=SingletonMeta):
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.suffixes = self.read(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'data', "suffix.txt"))

    def read(self, filename):
        result = set()
        if not os.path.exists(filename):
            self.logger.error("load suffixes from [%s] failed", filename)
            return result
        with open(filename, 'r') as sfile:
            for y in sfile:
                result.add(y.strip())
        return result

    def remove_suffix(self, text):
        while text and text[-1] in self.suffixes:
            text = text[:-1]
        return text


suffix_remover = SuffixRemover()


